@@ -1,7 +1,7 @@
 """spawning_child.py
 """
 
-from eventlet import api, coros, greenio, wsgi
+from eventlet import api, coros, greenio, tpool, wsgi
 
 import optparse, os, signal, socket, sys, time
 
@@ -27,10 +27,8 @@ class ExecuteInThreadpool(object):
         self.app = app
 
     def __call__(self, env, start_response, exc_info=None):
-        from eventlet import tpool
         head = []
         body = []
-        exc_info = [None]
         def _start_response(status, headers, exc_info=None):
             head[:] = status, headers, exc_info
             return body.append
@@ -43,24 +41,22 @@ class ExecuteInThreadpool(object):
         return result
 
 
-def serve_from_child(sock, base_dir, config_url, global_conf, threads):
-    wsgi_application = loadwsgi.loadapp(
-        config_url, relative_to=base_dir, global_conf=global_conf)
+def serve_from_child(sock, config):
+    wsgi_application = api.named(config['app_factory'])(config)
 
+    threads = config.get('threadpool_workers', 0)
     if threads:
         print "(%s) using %s threads" % (os.getpid(), threads, )
-        os.environ['EVENTLET_THREADPOOL_SIZE'] = str(threads)
-        from eventlet import tpool
         wsgi_application = ExecuteInThreadpool(wsgi_application)
     else:
+        print "(%s) not using threads, installing eventlet cooperation monkeypatching" % (
+            os.getpid(), )
         from eventlet import util
         util.wrap_socket_with_coroutine_socket()
         util.wrap_pipes_with_coroutine_pipes()
         util.wrap_threading_local_with_coro_local()
 
     host, port = sock.getsockname()
-    print "(%s) wsgi server listening on %s:%s using %s from %s (in %s)" % (
-        os.getpid(), host, port, wsgi_application, config_url, base_dir)
 
     server_event = coros.event()
     try:
@@ -71,6 +67,10 @@ def serve_from_child(sock, base_dir, config_url, global_conf, threads):
     except ExitChild:
         pass
 
+    ## Once we get here, we just need to handle outstanding sockets, not
+    ## accept any new sockets, so we should close the server socket.
+    sock.close()
+
     server = server_event.wait()
 
     last_outstanding = None
@@ -80,50 +80,50 @@ def serve_from_child(sock, base_dir, config_url, global_conf, threads):
                 os.getpid(), server.outstanding_requests)
         last_outstanding = server.outstanding_requests
         api.sleep(0.1)
-    print "(%s) Child exiting: all requests completed at %s" % (
+    print "(%s) *** Child exiting: all requests completed at %s" % (
         os.getpid(), time.asctime())
 
 
 def main():
+    from spawning import spawning_controller # spawning_controller just to watch for changes
     parser = optparse.OptionParser()
-    parser.add_option("-d", "--dev",
-        action='store_true', dest='dev',
-        help='If --dev is passed, reload the server any time '
-        'a loaded module changes. Otherwise, only when the svn '
-        'revision of the current directory changes.')
-    parser.add_option("-t", "--threads",
-        type='int', dest='threads', default=0,
-        help='If --threads is passed, use a threadpool for executing '
-            'individual wsgi applications. If not, just run the wsgi '
-            'application in the io thread and install the eventlet '
-            'cooperative socket object so that i/o operations will '
-            'automatically yield to another wsgi application.')
+    parser.add_option("-r", "--reload",
+        action='store_true', dest='reload',
+        help='If --reload is passed, reload the server any time '
+        'a loaded module changes.')
 
     options, args = parser.parse_args()
 
-    if len(args) < 6:
-        print "Usage: %s controller_pid config_url base_dir httpd_fd death_fd global_conf" % (
+    if len(args) != 5:
+        print "Usage: %s controller_pid httpd_fd death_fd factory_qual factory_args" % (
             sys.argv[0], )
         sys.exit(1)
 
-    controller_pid, config_url, base_dir, httpd_fd, death_fd, global_conf = args
+    controller_pid, httpd_fd, death_fd, factory_qual, factory_args = args
     controller_pid = int(controller_pid)
-    global_conf = simplejson.loads(global_conf)
+    config = api.named(factory_qual)(simplejson.loads(factory_args))
 
     ## Set up the reloader
-    if options.dev:
+    if options.reload:
+        watch = config.get('watch', None)
+        if watch:
+            watching = ' and %s' % watch
+        else:
+            watching = ''
+        print "(%s) reloader watching sys.modules%s" % (os.getpid(), watching)
         api.spawn(
-            reloader_dev.watch_forever, [], controller_pid, 1)
+            reloader_dev.watch_forever, [], controller_pid, 1, config.get('watch', []))
 
     ## The parent will catch sigint and tell us to shut down
     signal.signal(signal.SIGINT, signal.SIG_IGN)
     api.spawn(read_pipe_and_die, int(death_fd), api.getcurrent())
 
+    ## Make the socket object from the fd given to us by the controller
     sock = greenio.GreenSocket(
         socket.fromfd(int(httpd_fd), socket.AF_INET, socket.SOCK_STREAM))
 
     serve_from_child(
-        sock, base_dir, config_url, global_conf, options.threads)
+        sock, config)
 
 
 if __name__ == '__main__':
